using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Text;
using System.Threading;

using LMVCallbackLib;
using Leadtools;
using Leadtools.Codecs;
using Leadtools.Forms;
using Leadtools.Forms.Ocr;
using Leadtools.Forms.DocumentWriters;
using Leadtools.ImageProcessing.Core;

namespace WindowsApplication1
{
    class LMVMyUserCallback : ILMVUserCallback
    {
        private string TARGET_FILE = Application.StartupPath + @"\images\still{0:D5}.bmp";

        private bool m_bSnapshot;
        private volatile int m_nSnapshotCount = 0;
        private IOcrEngine ocrEngine;
        int processCount = 0;
        public LMVMyUserCallback()
        {
            ocrEngine = OcrEngineManager.CreateEngine(OcrEngineType.Advantage, false);
            //start out OCR engine
            ocrEngine.Startup(null, null, null, null);
        }
        public bool TakeSnapshot
        {
            get
            {
                return m_bSnapshot;
            }
            set
            {
                m_bSnapshot = value;
            }
        }
        public int SnapshotCount
        {
            get
            {
                return m_nSnapshotCount;
            }
            set
            {
                m_nSnapshotCount = value;
            }
        }


        public void ReceiveProc(int pData, int lWidth, int lHeight, int lBitCount, int lSize, int bTopDown)
        {
          
            try
            {
                if (m_bSnapshot)
                {
                    // Get the background image
                    RasterViewPerspective viewPerspective;
                    if (bTopDown == 1)
                        viewPerspective = RasterViewPerspective.TopLeft;
                    else
                        viewPerspective = RasterViewPerspective.BottomLeft;

                    RasterImage img = new RasterImage(
                        RasterMemoryFlags.User, //A combination of the RasterMemoryFlags enumeration members indicating the type of memory to allocate for the image data.
                        lWidth, // Width of the image in pixels.
                        lHeight, //Height of the image in pixels.
                        lBitCount, //The number of bits per pixel.
                        RasterByteOrder.Bgr, //Color order for 16-, 24-, 32-, 48- and 64-bit images.
                        viewPerspective, //Specifies where the beginning of the image is stored.
                        null, //The palette that the image will use. You can specify your own palette, or use null (Nothing in Visual Basic) for LEAD's fixed palette. The palette member is used only when bitsPerPixel is less than or equal to 8.
                        IntPtr.Zero, //Unmanaged data pointer that will contain the image data when flags is RasterMemoryFlags.User.
                        0); //Length in bytes of the data passed to userData. Only when used when userData is not IntPtr.Zero and flags is RasterMemoryFlags.User.

                    img.SetUserData(new IntPtr(pData), lSize);
                    RasterImage processedImg = new RasterImage(img);
                    string pdfFileName = "";
                    pdfFileName = string.Format(TARGET_FILE.Substring(0, TARGET_FILE.Length - 3), m_nSnapshotCount) + "pdf";

                    //we want to OCR in a separate thread so we don't lock up the program
                    Thread processThread = new Thread(() => processOcr(img, pdfFileName));
                    processThread.Start();

                    // Save the still frame as a bmp
                    RasterCodecs codecs = new RasterCodecs();
                    codecs.Save(img, string.Format(TARGET_FILE, m_nSnapshotCount), RasterImageFormat.Bmp, 24);
                    m_bSnapshot = false;
                    m_nSnapshotCount++;
                    processCount++;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        public void processOcr(RasterImage processedImg, string filename)
        {
            using (IOcrDocument ocrDocument = ocrEngine.DocumentManager.CreateDocument())
            {
                
                //autobinarize the image to help with the OCR results
                AutoBinarizeCommand cmd = new AutoBinarizeCommand();
                cmd.Run(processedImg);
                //add additional pre processing here if you need to improve the OCR results

                //common command that helps:
                //MinimumCommand cmd2 = new MinimumCommand();
                //cmd2.Dimension = 2;
                //cmd2.Run(processedImg);

                //add a page to our document
                IOcrPage ocrPage = ocrDocument.Pages.AddPage(processedImg, null);
                //autozone
                ocrPage.AutoZone(null);
                // Recognize it and save it as PDF
                ocrPage.Recognize(null);
                ocrDocument.Save(filename, DocumentFormat.Pdf, null);

            }
        }
    }
}
